package bc_global;

use strict;
use warnings;

use lib './cpan';

use Socket;
use Configfile::Ini;

sub set_defaults {
  our ($cname, $cpass, $server, $port, $TotalPacketsSent, $cackIDc, $initEncryption, $state, $TotalPacketsRecv, $sackIDc, $ackIDc, $serverKey, $chat, $debug, @pmodules, $mod, $posX, $posY);
  my ($setFile, $settings);

  # Get parameters
  if (length($ARGV[0]) > 0) {
    if ($ARGV[0] =~ /.*\.ini$/) {
      $setFile = "./settings/$ARGV[0]";
    }
  } else {
    $setFile = './settings/default.ini';
  }

  # Establish INI file
  $settings = new Configfile::Ini($setFile);

  $cname = $settings->get_entry_setting('client', 'name');
  $cpass = $settings->get_entry_setting('client', 'password');
  $server = $settings->get_entry_setting('server', 'serverIP');
  $port = $settings->get_entry_setting('server', 'serverPort');
  $mod = $settings->get_entry_setting('main', 'modules');
  $chat = $settings->get_entry_setting('main', 'chat');
  $debug = $settings->get_entry_setting('main', 'debug');
  bc_modules::req_mods();

  ## Initial ##

  # Client variable defaults
  $TotalPacketsSent = 0;
  $cackIDc = 0;
  $initEncryption = 0;
  #state: 0: Connecting  1: Running  2: Disconnected
  $state = 0;
  $posX = 8000;
  $posY = 8000;

  # Server variable defaults
  $TotalPacketsRecv = 0;
  $sackIDc = 0;
  $ackIDc = 0;
  $serverKey = 0;

}

sub start_up {
  # Get socket handle
  our ($socketfd, $server, $port); 
  socket($socketfd, PF_INET, SOCK_DGRAM, getprotobyname("udp")) or die "socket: $!";
  print "Shawn Bot 1.0\n";
  print "Server: $server\n";
  print "Port: $port\n";
}

# Signal EOF
1;
