package bc_packetgen;

use bc_enc;
use bc_handleproto;
use bc_global;

use strict;

# **Client to Server**

# 00 01
sub send_enc_request {

  # Offset	Length		Description
  # 0		2		Type Byte 0x00,0x01
  # 2		4		Client Encryption Key
  # 6		2		Client Version/Type
 
  my @pclientKey;
  my ($key, $ck);
  $ck = int(-rand() * 2147483647);
  print "Client key: $ck\n";
  
  $pclientKey[0] = bc_enc::enc_byte(0x00);
  $pclientKey[1] = bc_enc::enc_byte(0x01);
  @pclientKey[2 .. 5] = bc_enc::ence_nint($ck);
  $pclientKey[6] = bc_enc::enc_byte(0x01);
  $pclientKey[7] = bc_enc::enc_byte(0x00);
  $key = bc_enc::encrypt(@pclientKey);
  $bc_global::clientKey = $ck;
  bc_handleproto::C2S_send($key);
  bc_handleproto::console_debug("enc request sent: @pclientKey");
}

sub send_enc_request_noenc {
 
  my @pclientKey;
  my ($key, $ck);
  $ck = 0;
  
  $pclientKey[0] = bc_enc::enc_byte(0x00);
  $pclientKey[1] = bc_enc::enc_byte(0x01);
  @pclientKey[2 .. 5] = bc_enc::ence_nint($ck);
  $pclientKey[6] = bc_enc::enc_byte(0x01);
  $pclientKey[7] = bc_enc::enc_byte(0x00);
  $key = bc_enc::encrypt(@pclientKey);
  $bc_global::clientKey = $ck;
  bc_handleproto::C2S_send($key);
  bc_handleproto::console_debug("enc request sent: ".join(" ", bc_enc::tohex(@pclientKey)));
}

# 00 03
sub send_reliable_packet {
  # Offset	Length	Description
  # 0		1		Type Byte 0x03
  # 1		4		ACK ID
  # 5		*		Reliable Packet...
 
  my @packet = @_;
  my (@spacket, $rpacket);
  $spacket[0] = bc_enc::enc_byte(0x00);
  $spacket[1] = bc_enc::enc_byte(0x03);
  @spacket[2 .. 5] = bc_enc::ence_int($bc_global::cackIDc);
  push(@spacket, @packet);
  $rpacket = bc_enc::encrypt(@spacket);
  bc_handleproto::C2S_send($rpacket);
  $bc_global::clackID[$bc_global::cackIDc] = $rpacket;
  $bc_global::cackIDc++;
  bc_handleproto::console_debug("Sent reliable: @spacket");
}

sub resend_unacked_msgs {
  foreach (@bc_enc::clackID) {
    if ($_ != 1) {
      bc_handleproto::C2S_send($_);
      bc_handleproto::console_debug("resent: ".join(" ", bc_enc::decrypt($_)));
    }
  }
}

sub got_ack {
  my $aID = $_[0];
  my @t = (1);
  splice(@bc_enc::clackID, $aID, 1, @t);
}

# 00 04
sub send_reliable_ack {

  # Offset	Length	Description
  # 0		1		Type Byte 0x04
  # 1		4		ACK ID

  my $ackID = $_[0];
  my (@spacket, $rpacket);
  $spacket[0] = 0x00;
  $spacket[1] = 0x04;
  @spacket[2 .. 5] = bc_enc::ence_int($ackID);
  $rpacket = bc_enc::encrypt(@spacket);
  bc_handleproto::C2S_send($rpacket);
  bc_handleproto::console_debug("reliable ack sent: ".join(" ", bc_enc::tohex(@spacket)));
}

# 00 05
sub send_sync_request {
  
  # Offset	Length		Description
  # 0		2		Type Byte 0x00,0x05
  # 2		4		Current Ticks (Machine Up Time in MS Divided by 10)
  # 6		4		Total number of Packets Sent since Connnection
  # 10		4		Total number of Packets Recieved since Connection

  my @spacket;
  my ($rpacket, $ticks);
  $ticks = int(bc_handleproto::getTicks() / 10);
  $spacket[0] = bc_enc::enc_byte(0x00);
  $spacket[1] = bc_enc::enc_byte(0x05);
  @spacket[2 .. 5] = bc_enc::ence_int($ticks);
  @spacket[6 .. 9] = bc_enc::ence_int($bc_global::TotalPacketsSent);
  @spacket[10 .. 13] = bc_enc::ence_int($bc_global::TotalPacketsRecv);
  $rpacket = bc_enc::encrypt(@spacket);
  bc_handleproto::C2S_send($rpacket);
  bc_handleproto::console_debug("sync request sent: ".join(" ", bc_enc::tohex(@spacket)));
}

# 00 06
sub send_sync_response {
  # Offset	Length	Descriptoin
  # 0		2		Type Byte 0x00,0x06
  # 2		4		Servers Time Stamp
  # 6		4		Current Ticks (Machine Up Time in MS Divided by 10)

  my @spacket;
  my ($timeStamp, $ticks, $rpacket);
  $ticks = int(bc_handleproto::getTicks() / 10);
  $timeStamp = $_[0];
  $spacket[0] = bc_enc::enc_byte(0x00);
  $spacket[1] = bc_enc::enc_byte(0x06);
  @spacket[2 .. 5] = bc_enc::ence_int($timeStamp);
  @spacket[6 .. 9] = bc_enc::ence_int($ticks);
  $rpacket = bc_enc::encrypt(@spacket);
  #print "server ts: $timeStamp\n";
  #print "ticks: $ticks\n";
  bc_handleproto::C2S_send($rpacket);
  bc_handleproto::console_debug("sync response sent: ".join(" ", bc_enc::tohex(@spacket)));

}

# 00 07
sub send_disconnect {
  my (@spacket, $rpacket);
  $spacket[0] = bc_enc::enc_byte(0x00);
  $spacket[1] = bc_enc::enc_byte(0x07);
  $rpacket = bc_enc::encrypt(@spacket);
  bc_handleproto::C2S_send($rpacket);
  print "disconnected\n";
}

# 00 0E
sub send_cluster {
  # Offset	Length	Description
  # 0		1		Type Byte 0x0F
  # The following are repeated until the end of the message
  # >1		1		Packet Length
  # >2		*		Packet...

}

# Standard Packets

# 01
sub send_arena_login {

  # Offset	Length	Description
  # 0		1		Type Byte 0x01
  # 1		1		Ship type *1
  # 2		2		Allow audio *2
  # 4		2		X Resolution
  # 6		2		Y Resolution
  # 8		2		Main arena number *3
  # 10		16		Arena name	(Optional)
 
  my (@spacket, $rpacket, $public, $sub, @arena);
  ($public, $sub) = @_;
  @arena = bc_enc::pad_array(16);
  @arena[0 .. length($sub)-1] = unpack("C*", $sub);

  $spacket[0] = bc_enc::enc_byte(0x01);
  $spacket[1] = bc_enc::enc_byte(0x08);
  @spacket[2 .. 3] = bc_enc::ence_short(bc_enc::short(0));
  @spacket[4 .. 5] = bc_enc::ence_short(bc_enc::short(1280));
  @spacket[6 .. 7] = bc_enc::ence_short(bc_enc::short(1024));
  @spacket[8 .. 9] = bc_enc::ence_short(bc_enc::short($public));
  #@spacket[8 .. 9] = bc_enc::ence_short(bc_enc::short(0xFF));
  @spacket[10 .. 26] = @arena;

  #$rpacket = bc_enc::encrypt(@spacket);
  bc_handleproto::console_debug("arena: @spacket");
  send_reliable_packet(@spacket);
  bc_handleproto::console_write("Entering public arena $sub");
}

# 02
sub send_leave_arena {
  # Offset	Length	Description
  # 0		1		Type Byte 0x02

  my @spacket;

  $spacket[0] = bc_enc::enc_byte(0x02);

  #$rpacket = bc_enc_encrypt(@spacket);
  send_reliable_packet(@spacket);
  bc_handleproto::console_write("Leaving arena"); 
}

# 03
sub send_position {
  # Offset	Length	Description
  # 0		1		Type Byte 0x03
  # 1		1		Direction	(0 ... 360)
  # 2		4		Timestamp
  # 6		2		X Velocity
  # 8		2		Y Pixels	(0 ... 16384)
  # 10		1		Checksum
  # 11		1		Togglables *1
  # 12		2		X Pixels	(0 ... 16384)
  # 14		2		Y Velocity
  # 16		2		Bounty
  # 18		2		Energy
  # 20		2		Weapon Info *2
  # 
  # 22		2		Energy *4		(Optional)
  # 24		2		S2C Latency *4	(Optional)
  # 26		2		Timer *4		(Optional)
  # 28		4		Item info *3 *4	(Optional)

  my (@spacket, $rpacket, $ticks, $csum, $i);
  $ticks = int(bc_handleproto::getTicks() / 10);

  $spacket[0] = bc_enc::enc_byte(0x03);
  $spacket[1] = bc_enc::enc_byte(0x00);
  @spacket[2 .. 5] = bc_enc::ence_int($ticks);
  @spacket[6 .. 7] = bc_enc::ence_short(bc_enc::short(0x00));
  @spacket[8 .. 9] = bc_enc::ence_short(bc_enc::short($bc_global::posY));
  $spacket[10] = bc_enc::enc_byte(0x00);
  $spacket[11] = bc_enc::enc_byte(0x00);
  @spacket[12 .. 13] = bc_enc::ence_short(bc_enc::short($bc_global::posX));
  @spacket[14 .. 15] = bc_enc::ence_short(bc_enc::short(0x00));
  @spacket[15 .. 16] = bc_enc::ence_short(bc_enc::short(0x00));
  @spacket[17 .. 18] = bc_enc::ence_short(bc_enc::short(0x00));
  @spacket[19 .. 20] = bc_enc::ence_short(bc_enc::short(0x00));
 
  for ($i = 0; $i < scalar(@spacket); $i++) {
    $csum = $csum ^ $spacket[$i];
  }
  $spacket[10] = bc_enc::enc_byte($csum);
  $rpacket = bc_enc::encrypt(@spacket);
  bc_handleproto::C2S_send($rpacket);
}

# 04 Unknown

# 05
sub send_death_message {
  # Offset	Length	Description
  # 0		1		Type Byte 0x05
  # 1		2		Killer's Player ID
  # 3		2		Your bounty at time of death

  my @spacket;
  my ($killerID, $bounty) = @_;

  $spacket[0] = enc_enc::enc_byte(0x05);
  @spacket[1 .. 2] = enc_enc::ence_short($killerID);
  @spacket[3 .. 4] = enc_enc::ence_short($bounty);

  send_reliable_packet(@spacket);
}

# 06
sub send_chat_message {
  
  # Offset	Length	Description
  # 0		1		Type Byte 0x06
  # 1		1		Chat Type *1
  # 2		1		Sound Byte
  # 3		2		Target Player's Player ID *2
  # 5		*		Text...
  # 
  # Chat types:
  # 0x00 - Message in green text  [*arena, *zone, ...]
  # 0x01 - Public macro
  # 0x02 - Public message
  # 0x03 - Team message  [// or ']
  # 0x04 - Player to all members of another team  ["Whatever]
  # 0x05 - Private message  [/Whatever]
  # 0x06 - Red warning message  [MODERATOR WARNING: Whatever -Whoever]
  # 0x07 - Remote private message	*3  [(Whoever)> Whatever]
  # 0x08 - Red server errors, without a name tag  (S2C only)
  # 0x09 - Channel message *3  [X;Whatever]

  use Switch;
  
  my ($type, $targetid, $message) = @_;
  my (@spacket, $rpacket);

  $spacket[0] = bc_enc::enc_byte(0x06);
  $spacket[1] = bc_enc::enc_byte($type);
  $spacket[2] = bc_enc::enc_byte(0x00);
  
  switch ($type) {
    case 0x00 {
      @spacket[3 .. 4] = bc_enc::ence_short(0x00);
      @spacket[5 .. 5+length($message)] = unpack("C*", $message);
    }
    case 0x01 {
      @spacket[3 .. 4] = bc_enc::ence_short(0x00);
      @spacket[5 .. 5+length($message)] = unpack("C*", $message);
    }
    case 0x02 {
      @spacket[3 .. 4] = bc_enc::ence_short(0x00);
      @spacket[5 .. 5+length($message)] = unpack("C*", $message);
    }
    case 0x03 {
      @spacket[3 .. 4] = bc_enc::ence_short(0x00);
      @spacket[5 .. 5+length($message)] = unpack("C*", $message);
    }
    case 0x04 { # $targetid == freq number
      @spacket[3 .. 4] = bc_enc::ence_short($targetid);
      @spacket[5 .. 5+length($message)] = unpack("C*", $message);
    }
    case 0x05 { # $targetid == playerid
      @spacket[3 .. 4] = bc_enc::ence_short($targetid);
      @spacket[5 .. 5+length($message)] = unpack("C*", $message);
    }
    case 0x06 {
      @spacket[3 .. 4] = bc_enc::ence_short(0x00);
      @spacket[5 .. 5+length($message)] = unpack("C*", $message);
    }
    case 0x07 { # $targetid == player name
      @spacket[3 .. 4] = bc_enc::ence_short(0x00);
      $message = ":$targetid",":$message";
      @spacket[5 .. 5+length($message)] = unpack("C*", $message);
    }
    case 0x08 {
      @spacket[3 .. 4] = bc_enc::ence_short(0x00);
      @spacket[5 .. 5+length($message)] = unpack("C*", $message);
    }
    case 0x09 { # $targetid == chat number
      @spacket[3 .. 4] = bc_enc::ence_short(0x00);
      $message = $targetid,";$message";
      @spacket[5 .. 5+length($message)] = unpack("C*", $message);
    }
    else {
    }
  }
  send_reliable_packet(@spacket);
}

# 07 Unknown

# 08
sub send_spec_player {
  # Offset	Length	Description
  # 0		1		Type Byte 0x08
  # 1		2		Spectated Player's Player ID

  my @spacket;
  my ($playerID) = @_;

  $spacket[0] = bc_enc::enc_byte(0x08);
  @spacket[1 .. 2] = bc_enc::ence_short($playerID);

  send_reliable_packet(@spacket);
}

# 09
sub send_password {

  # Offset	Length		Description
  # 0		1		Type byte
  # 1		1		Boolean: New user *1
  # 2		32		Name
  # 34		32		Password
  # 66		4		Machine ident *2
  # 70		1		ConnectType *3
  # 71		2		Timezone bias
  # 73		2		?
  # 75		2		Client version *4
  # 77		4		? memory checksum, Set to = 444
  # 81		4		? memory checksum, Set to = 555
  # 85		4		Permission ident
  # 89		12		?
  # 
  # *1 - 1 = New, 0 = Not New
  # 
  # *2 - Should be Drive Seriel of C (can be random for bots)
  # 
  # *3 - Type 0x00 is a safe bet.
  # 
  # *4 - 0x24 = Ctm, 0x86 = SS

  my ($name, $pass) = @_;
  my @ppacket;
  my $ppacketConv;
  @ppacket = bc_enc::pad_array(101);

  # Packet start

  $ppacket[0] = bc_enc::enc_byte(0x09);
  $ppacket[1] = bc_enc::enc_byte(0x00);
  @ppacket[2 .. 2+length($name)-1] = unpack("C*", $name);
  @ppacket[34 .. 34+length($pass)-1] = unpack("C*", $pass);

  # Machine ID
  $ppacket[66] = bc_enc::enc_byte(sprintf("%x", int(rand(245)) + 10));
  $ppacket[67] = bc_enc::enc_byte(sprintf("%x", int(rand(245)) + 10));
  $ppacket[68] = bc_enc::enc_byte(sprintf("%x", int(rand(245)) + 10));
  $ppacket[69] = bc_enc::enc_byte(sprintf("%x", int(rand(245)) + 10));

  $ppacket[70] = bc_enc::enc_byte(0x00);

  # Timezone Bias
  @ppacket[71 .. 72] = bc_enc::ence_short(bc_enc::short(240));

  @ppacket[73 .. 74] = bc_enc::ence_short(bc_enc::short(0x6f9d));
  @ppacket[75 .. 76] = bc_enc::ence_short(bc_enc::short(0x86));
  @ppacket[77 .. 80] = bc_enc::ence_int(444);
  @ppacket[81 .. 84] = bc_enc::ence_int(555);

  # Permission ID
  $ppacket[85] = bc_enc::enc_byte(sprintf("%x", int(rand(245)) + 10));
  $ppacket[86] = bc_enc::enc_byte(sprintf("%x", int(rand(245)) + 10));
  $ppacket[87] = bc_enc::enc_byte(sprintf("%x", int(rand(245)) + 10));
  $ppacket[88] = bc_enc::enc_byte(sprintf("%x", int(rand(245)) + 10));

  $ppacket[89] = bc_enc::enc_byte(0x00);
  $ppacket[90] = bc_enc::enc_byte(0x00);
  $ppacket[91] = bc_enc::enc_byte(0x00);
  $ppacket[92] = bc_enc::enc_byte(0x00);
  $ppacket[93] = bc_enc::enc_byte(0x00);
  $ppacket[94] = bc_enc::enc_byte(0x00);
  $ppacket[95] = bc_enc::enc_byte(0x00);
  $ppacket[96] = bc_enc::enc_byte(0x00);
  $ppacket[97] = bc_enc::enc_byte(0x00);
  $ppacket[98] = bc_enc::enc_byte(0x00);
  $ppacket[99] = bc_enc::enc_byte(0x00);
  $ppacket[100] = bc_enc::enc_byte(0x00);

  # Packet end

  send_reliable_packet(@ppacket);
  bc_handleproto::console_debug("password sent: ".join(" ", bc_enc::tohex(@ppacket)));
}

# 0A Unknown

# 0B
sub send_ssupdate_request {
  # Offset	Length	Description
  # 0		1		Type Byte 0x0B

  my @spacket;

  $spacket[0] = bc_enc::enc_byte(0x0B);

  send_reliable_packet(@spacket);
}

# 0C
sub send_map_request {
  # Offset	Length	Description
  # 0		1		Type Byte 0x0C

  my @spacket;

  $spacket[0] = bc_enc::enc_byte(0x0C);

  send_reliable_packet(@spacket);
}

# 0D
sub send_news_request {
  # Offset	Length	Description
  # 0		1		Type Byte 0x0D

  my @spacket;

  $spacket[0] = bc_enc::enc_byte(0x0D);

  send_reliable_packet(@spacket);
}

# 0E send_voice_message # Not implemented

# 0F
sub send_freq_change {
  # Offset	Length	Description
  # 0		1		Type Byte 0x0F
  # 1		2		New Frequency

  my @spacket;
  my ($freq) = @_;

  $spacket[0] = bc_enc::enc_byte(0x0F);
  @spacket[1 .. 2] = bc_enc::ence_short($freq);

  send_reliable_packet(@spacket);
}

# 10
sub send_attach_request {
  # Offset	Length	Description
  # 0		1		Type Byte 0x10
  # 1		2		Player ID of Turretee

  my @spacket;
  my ($playerID) = @_;

  $spacket[0] = bc_enc::enc_byte(0x10);
  @spacket[1 .. 2] = bc_enc::ence_short($playerID);

  send_reliable_packet(@spacket);
}

# 11 Unknown

# 12 Unknown

# 13
sub send_flag_request {
  # Offset	Length	Description
  # 0		1		Type Byte 0x13
  # 1		2		Flag ID

  my @spacket;
  my ($flagID) = @_;

  $spacket[0] = bc_enc::enc_byte(0x13);
  @spacket[1 .. 2] = bc_enc::ence_short($flagID);

  send_reliable_packet(@spacket);
}

# 14 Unknown

# 15
sub send_drop_flags {
  # Offset	Length	Description
  # 0		1		Type Byte 0x15

  my @spacket;

  $spacket[0] = bc_enc::enc_byte(0x15);

  send_reliable_packet(@spacket);
}

# 16 send_file_transfer #Not implemented

# 17 send_regform_response #Not implemented

# 18
sub send_set_shiptype {
  # Offset	Length	Description
  # 0		1		Type Byte 0x18
  # 1		1		Ship Type *1
 
  my @spacket;
  my ($shiptype) = @_;

  $spacket[0] = bc_enc::enc_byte(0x18);
  $spacket[1] = bc_enc::enc_byte($shiptype);

  send_reliable_packet(@spacket);
}

# 19 send_set_banner #Not implemented

# 1A send_security_checksum #Not implemented

# 1B
sub security_violation {
  # Offset	Length	Description
  # 0		1		Type Byte 0x1B
  # 1		1		Violation ID *1

  my @spacket;
  my ($violationID) = @_;

  $spacket[0] = bc_enc::enc_byte(0x1B);
  $spacket[1] = bc_enc::enc_byte($violationID);

  send_reliable_packet(@spacket);
}

# 1C
sub send_drop_brick {
  # Offset	Length	Description
  # 0		1		Type Byte 0x1C
  # 1		2		X Tiles
  # 3		2		Y Tiles

  my @spacket;
  my ($xTiles, $yTiles) = @_;

  $spacket[0] = bc_enc::enc_byte(0x1C);
  @spacket[1 .. 2] = bc_enc::ence_short($xTiles);
  @spacket[1 .. 2] = bc_enc::ence_short($yTiles);

  send_reliable_packet(@spacket);
}

# 1D Unknown

# 1E
sub send_end_koth_timer {
  # Offset	Length	Description
  # 0		1		Type Byte 0x1E

  my @spacket;

  $spacket[0] = bc_enc::enc_byte(0x1E);

  send_reliable_packet(@spacket);
}

# 1F
sub send_fire_ball  {
  # Offset	Length	Description
  # 0		1		Type Byte 0x1F
  # 1		1		Ball ID
  # 2		2		X Pixels
  # 4		2		Y Pixels
  # 6		2		X Velocity
  # 8		2		Y Velocity
  # 10		2		Your Player ID
  # 12		4		Timestamp

  my @spacket;
  my ($ballID, $xPixels, $yPixels, $xVelocity, $yVelocity, $playerID, $timeStamp) = @_;

  $spacket[0] = bc_enc::enc_byte(0x1F);
  $spacket[1] = bc_enc::enc_byte($ballID);
  @spacket[2 .. 3] = bc_enc::ence_short($xPixels);
  @spacket[4 .. 5] = bc_enc::ence_short($yPixels);
  @spacket[6 .. 7] = bc_enc::ence_short($xVelocity);
  @spacket[8 .. 9] = bc_enc::ence_short($yVelocity);
  @spacket[10 .. 11] = bc_enc::ence_short($playerID);
  @spacket[12 .. 15] = bc_enc::ence_int($timeStamp);

  send_reliable_packet(@spacket);
}

# 20
sub send_ball_request {
  # Offset	Length	Description
  # 0		1		Type Byte 0x20
  # 1		1		Ball ID
  # 2		4		Timestamp

  my @spacket;
  my ($ballID, $timeStamp) = @_;

  $spacket[0] = bc_enc::enc_byte(0x20);
  $spacket[1] = bc_enc::enc_byte($ballID);
  @spacket[2 .. 5] = bc_enc::ence_int($timeStamp);

  send_reliable_packet(@spacket);
}

# 21 Unknown

# 22 Unknown

# -- Continuum packets

# 23 Unknown

# 24 send_continuum_password #Not implemented

# Signal EOF
1;
