# Packet Interpreter

package sample;

use bc_enc;
use bc_handleproto;
use bc_permission;
use bc_global;

use strict;
use Switch;

# This function runs every second
sub event {
}

sub pi {
  my @packet = @_;

  if ($packet[0] eq "second") {
    event();
    return 0;
  }

  if ($packet[0] eq "undef_p") {
    our (@p, @ids);
    undef @p;
    undef @ids;
    return 0;
  }

  switch ($packet[0]) {
    case 0x01	{ # Notification of the Bots ID
      my @packet = @_;
      my $botid;

      $botid = bc_enc::dece_short(@packet[1 .. 2]);
    }

    case 0x02	{ # Player is In the Game
      # You are now in the game
    }

    case 0x03	{ # Player(s) entering the Game
      my @packet = @_;
      my ($playerID, $count, $i);
      our (@p, @ids, $started);

      $count = 0;
      $i = 0;
      while(1) {
	$playerID = bc_enc::dece_short(@packet[$count+51 .. $count+52]);
	unless(grep {$_ == $playerID} @ids) {
	  push(@ids, $playerID);
	}

	$p[$playerID]{'ship_type'} = @packet[$count+1];
	$p[$playerID]{'name'} = unpack("Z*", pack("C*", @packet[$count+3 .. $count+22]));
	$p[$playerID]{'squad'} = unpack("Z*", pack("C*", @packet[$count+23 .. $count+42]));
	$p[$playerID]{'flag_points'} = bc_enc::dece_int(@packet[$count+43 .. $count+46]);
	$p[$playerID]{'kill_points'} = bc_enc::dece_int(@packet[$count+47 .. $count+50]);
	$p[$playerID]{'user_id'} = bc_enc::dece_short(@packet[$count+51 .. $count+52]);
	$p[$playerID]{'wins'} = bc_enc::dece_short(@packet[$count+53 .. $count+54]);
	$p[$playerID]{'losses'} = bc_enc::dece_short(@packet[$count+55 .. $count+56]);
	$p[$playerID]{'audio'} = join(" ", @packet[$count+57 .. $count+62]);

	if (@packet[$count+64] != 03) {
	  last;
	}
	$count += 64;
	$i++;
      }
    }

    case 0x04	{ # Player Leaving the game
      my @packet = @_;
      my ($playerID, $i);
      our (@p, @ids);

      $playerID = bc_enc::dece_short(@packet[1 .. 2]); 

      splice(@p, $playerID, 1, undef);
      for ($i = 0; $i < scalar(@ids); $i++) {
	if ($ids[$i] == $playerID) {
	  splice(@ids, $i, 1);
	}
      }
    }

    case 0x05	{ # Large position packet
      my @packet = @_;
      my $lastel;
      my ($direction, $timeStamp, $XPixels, $YVelocity, $playerID, $XVelocity, $checksum, $toggleables, $ping, $YPixels, $bounty, $weaponParameters, $weaponType, $energy, $lag, $timers, $items);

      $lastel = $#packet;

      $direction = $packet[1];
      $timeStamp = bc_enc::dece_short(@packet[2 .. 3]);
      $XPixels = bc_enc::dece_short(@packet[4 .. 5]);
      $YVelocity = bc_enc::dece_short(@packet[6 .. 7]);
      $playerID = bc_enc::dece_short(@packet[8 .. 9]);
      $XVelocity = bc_enc::dece_short(@packet[10 .. 11]);
      $checksum = $packet[12];
      $toggleables = $packet[13];
      $ping = $packet[14];
      $YPixels = bc_enc::dece_short(@packet[15 .. 16]);
      $bounty = bc_enc::dece_short(@packet[17 .. 18]);
      $weaponParameters = $packet[19];
      $weaponType = $packet[20];
      if ($lastel >= 21) {
	$energy = bc_enc::dece_short(@packet[21 .. 22]);
	$lag = bc_enc::dece_short(@packet[23 .. 24]);
	$timers = bc_enc::dece_short(@packet[25 .. 26]);
	$items = bc_enc::dece_int(@packet[27 .. 30]);
      }
    }

    case 0x06	{ # Player death
      my @packet = @_;
      my ($killerID, $killedID, $bounty);
      our @p;

      $killerID = bc_enc::dece_short(@packet[2 .. 3]);
      $killedID = bc_enc::dece_short(@packet[4 .. 5]);
      $bounty = bc_enc::dece_short(@packet[6 .. 7]);

      $p[$killerID]{'wins'} = int($p[$killerID]{'wins'} + 1);
      $p[$killedID]{'losses'} = int($p[$killedID]{'losses'} + 1);
    }

    case 0x07	{ # Recieved a Chat Message
      my @packet = @_;
      my ($player, $playerID, $message, $themsg, $adj);

      $adj = 2;
      switch($packet[1]) {
	case 0x00	{ # Green message
	  $playerID = bc_enc::dece_short(@packet[3 .. 4]);
	  $themsg = pack("C*", @packet[5 .. $#packet-$adj]);
	  $themsg =~ s/[^ -~]//g;

	}
	case 0x01	{ # Public macro
	  $playerID = bc_enc::dece_short(@packet[3 .. 4]);
	  $themsg = pack("C*", @packet[5 .. $#packet-$adj]);
	  $themsg =~ s/[^ -~]//g;
	  command(1, $playerID, $themsg);

	}
	case 0x02	{ # Public message
	  $playerID = bc_enc::dece_short(@packet[3 .. 4]);
	  $themsg = pack("C*", @packet[5 .. $#packet-$adj]);
	  $themsg =~ s/[^ -~]//g;
	  command(2, $playerID, $themsg);

	}
	case 0x03	{ # Team message
	  $playerID = bc_enc::dece_short(@packet[3 .. 4]);
	  $themsg = pack("C*", @packet[5 .. $#packet-$adj]);
	  $themsg =~ s/[^ -~]//g;
	  command(3, $playerID, $themsg);

	}
	case 0x04	{ # Enemy team message
	  $playerID = bc_enc::dece_short(@packet[3 .. 4]);
	  $themsg = pack("C*", @packet[5 .. $#packet-$adj]);
	  $themsg =~ s/[^ -~]//g;

	}
	case 0x05	{ # Private message
	  $playerID = bc_enc::dece_short(@packet[3 .. 4]);
	  $themsg = pack("C*", @packet[5 .. $#packet-$adj]);
	  $themsg =~ s/[^ -~]//g;
	  command(5, $playerID, $themsg);

	}
	case 0x06	{ # Moderator warning
	  $playerID = bc_enc::dece_short(@packet[3 .. 4]);
	  $themsg = pack("C*", @packet[5 .. $#packet-$adj]);
	  $themsg =~ s/[^ -~]//g;

	}
	case 0x07	{ # Remote private message
	  $playerID = bc_enc::dece_short(@packet[3 .. 4]); # Is not used
	  $themsg = pack("C*", @packet[5 .. $#packet-$adj]);
	  $themsg =~ s/[^ -~]//g;
	  $themsg =~ /\((.+)\)>(.+)/;
	  $player = $1;
	  $message = $2;

	}
	case 0x08	{ # Server error [S2C only]
	  $themsg = pack("C*", @packet[5 .. $#packet]);
	}
	case 0x09	{ # Channel message
	  $playerID = bc_enc::dece_short(@packet[3 .. 4]);
	  $themsg = pack("C*", @packet[5 .. $#packet-$adj]);
	  $themsg =~ s/[^ -~]//g;

	}
	else {
	}
      }
    }

    case 0x08	{ # Player prized
      my @packet = @_;
      my ($timeStamp, $XTiles, $YTiles, $prize, $playerID);

      $timeStamp = bc_enc::dece_int(@packet[1 .. 4]);
      $XTiles = bc_enc::dece_short(@packet[5 .. 6]);
      $YTiles = bc_enc::dece_short(@packet[7 .. 8]);
      $prize = bc_enc::dece_short(@packet[9 .. 10]);
      $playerID = bc_enc::dece_short(@packet[11 .. 12]);
    }

    case 0x09	{ #Player score update
      my @packet = @_;
      my ($playerID);
      our @p;

      $playerID = bc_enc::dece_short(@packet[1 .. 2]);
      $p[$playerID]{'flag_points'} = bc_enc::dece_int(@packet[3 .. 6]);
      $p[$playerID]{'kill_points'} = bc_enc::dece_int(@packet[7 .. 10]);
      $p[$playerID]{'wins'} = bc_enc::dece_short(@packet[11 .. 12]);
      $p[$playerID]{'losses'} = bc_enc::dece_short(@packet[13 .. 14]);
    }

    case 0x0A	{ # Recieved Password Response
      my @packet = @_;
      my ($response, $version, $reg, $news);

      $response = $packet[1];
      $version = bc_enc::dece_int(@packet[2 .. 5]);
      $reg = $packet[19];
      $news = bc_enc::dece_int(@packet[24 .. 27]);
      #bc_packetgen::send_arena_login(0xFFFD, "duel");
      bc_handleproto::undef_p();
    }

    case 0x0B	{ # Soccer goal
      my @packet = @_;
      my ($scoreFreq, $teamPoints);

      $scoreFreq = bc_enc::dece_short(@packet[1 .. 2]);
      $teamPoints = bc_enc::dece_short(@packet[3 .. 6]);
    }

    case 0x0C	{} # Player voice

    case 0x0D	{ # Freq change
      my @packet = @_;
      my ($playerID, $freq);
      our @p;

      $playerID = bc_enc::dece_short(@packet[1 .. 2]);
      $freq = bc_enc::dece_short(@packet[3 .. 4]);
      $p[$playerID]{'freq'} = $freq;
    }

    case 0x0E	{ # Turret link
      my @packet = @_;
      my ($turretReq, $turretDes);

      $turretReq = bc_enc::dece_short(@packet[1 .. 2]);
      $turretDes = bc_enc::dece_short(@packet[3 .. 4]);
    }

    case 0x0F	{} # Arena Settings Recieved (Comes in four parts)

    case 0x10	{} # File transfer

    case 0x11	{} # Unknown

    case 0x12	{ # Flag position
      my @packet = @_;
      my ($flagID, $XTiles, $YTiles, $ownerTeam);

      $flagID = bc_enc::dece_short(@packet[1 .. 2]);
      $XTiles = bc_enc::dece_short(@packet[3 .. 4]);
      $YTiles = bc_enc::dece_short(@packet[5 .. 6]);
      $ownerTeam = bc_enc::dece_short(@packet[7 .. 8]);
    }

    case 0x13	{ # Flag claim
      my @packet = @_;
      my ($flagID, $playerID);

      $flagID = bc_enc::dece_short(@packet[1 .. 2]);
      $playerID = bc_enc::dece_short(@packet[3 .. 4]);
    }

    case 0x14	{ # Flag victory
      my @packet = @_;
      my ($team, $jackpotPoints);

      $team = bc_enc::dece_short(@packet[1 .. 2]);
      $jackpotPoints = bc_enc::dece_int(@packet[3 .. 6]);
    }

    case 0x15	{ # Destroy turret link
      my @packet = @_;
      my $turretDriverID;

      $turretDriverID = bc_enc::dece_short(@packet[1 .. 2]);
    }

    case 0x16	{ # Drop flag
      my @packet = @_;
      my $carrierID;

      $carrierID = bc_enc::dece_short(@packet[1 .. 2]);
    }

    case 0x17	{} # Unknown

    case 0x18	{ # Recieved a Checksum Request
      my @packet = @_;
      my ($prizeSeed, $doorSeed, $timeStamp, $checksumKey);

      $prizeSeed = bc_enc::dece_int(@packet[1 .. 4]);
      $doorSeed = bc_enc::dece_int(@packet[5 .. 8]);
      $timeStamp = bc_enc::dece_int(@packet[9 .. 12]);
      $checksumKey = bc_enc::dece_int(@packet[13 .. 16]);
    }

    case 0x19	{} # Request file

    case 0x1A	{ # Reset score(s)
      my @packet = @_;
      my $playerID;
      our @p;

      $playerID = bc_enc::dece_short(@packet[1 .. 2]);
      if ($playerID == 0xFFFF) {
	foreach (@p) {
	  if (length($_->{'name'}) > 0) {
	    $_->{'wins'} = 0;
	    $_->{'losses'} = 0;
	    $_->{'flag_points'} = 0;
	    $_->{'kill_points'} = 0;
	  }
	}
      }
    }

    case 0x1B	{ # Personal ship reset
    }

    case 0x1C	{ # Put player in spectator mode
      my @packet = @_;
      my ($playerID, $extraPosReq);

      if (scalar(@packet) == 3) {
	$playerID = bc_enc::dece_short(@packet[1 .. 2]);
      } else {
	$extraPosReq = @packet[1];
      }
    }

    case 0x1D	{ # Player team and ship changed
      my @packet = @_;
      my ($playerID, $freq);
      our @p;

      $playerID = bc_enc::dece_short(@packet[2 .. 3]);
      $p[$playerID]{'ship_type'} = @packet[1];
      $freq = bc_enc::dece_short(@packet[4 .. 5]);
      $p[$playerID]{'freq'} = $freq;
    }

    case 0x1E	{ # Personal banner changed
    }

    case 0x1F	{} # Player banner changed

    case 0x20	{ # Collected prize
      my @packet = @_;
      my ($timeStamp, $XTiles, $YTiles, $prizeType, $playerID);

      $timeStamp = bc_enc::dece_int(@packet[1 .. 4]);
      $XTiles = bc_enc::dece_short(@packet[5 .. 6]);
      $YTiles = bc_enc::dece_short(@packet[7 .. 8]);
      $prizeType = bc_enc::dece_short(@packet[9 .. 10]);
      $playerID = bc_enc::dece_short(@packet[11 .. 12]);
    }

    case 0x21	{ # Brick dropped
      my @packet = @_;
      my ($size, $lastel);
      my ($X1Tile, $Y1Tile, $X2Tile, $Y2Tile, $team, $brickID, $timeStamp);

      $size = 0;
      $lastel = $#packet;
      while($lastel >= $size) {
	$X1Tile = bc_enc::dece_short(@packet[$size+1 .. $size+2]);
	$Y1Tile = bc_enc::dece_short(@packet[$size+3 .. $size+4]);
	$X2Tile = bc_enc::dece_short(@packet[$size+5 .. $size+6]);
	$Y2Tile = bc_enc::dece_short(@packet[$size+7 .. $size+8]);
	$team = bc_enc::dece_short(@packet[$size+9 .. $size+10]);
	$brickID = bc_enc::dece_short(@packet[$size+11 .. $size+12]);
	$timeStamp = bc_enc::dece_int(@packet[$size+13 .. $size+16]);
	$size += 16;
      }
    }

    case 0x22	{ # Turf flag update
      my @packet = @_;
      my $size;
      my $team;

      $size = 0;
      while($#packet >= $size) {
	$team = bc_enc::dece_short(@packet[$size+1 .. $size+2]);
	$size += 2;
      }
    }

    case 0x23	{ # Flag reward granted
      my @packet = @_;
      my ($size, $lastel);
      my ($team, $pointsAwarded);

      $size = 0;
      $lastel = $#packet;
      while($lastel >= $size) {
	$team = bc_enc::dece_short(@packet[$size+1 .. $size+2]);
	$pointsAwarded = bc_enc::dece_short(@packet[$size+3 .. $size+4]);
	$size += 4;
      }
    }

    case 0x24	{ # Speed game over
      my @packet = @_;
      my ($best, $myRank, $myScore, $p1Score, $p2Score, $p3Score, $p4Score, $p5Score, $p1ID, $p2ID, $p3ID, $p4ID, $p5ID);

      $best = @packet[1];
      $myRank = bc_enc::dece_short(@packet[2 .. 3]);
      $myScore = bc_enc::dece_int(@packet[4 .. 7]);
      $p1Score = bc_enc::dece_int(@packet[8 .. 11]);
      $p2Score = bc_enc::dece_int(@packet[12 .. 15]);
      $p3Score = bc_enc::dece_int(@packet[16 .. 19]);
      $p4Score = bc_enc::dece_int(@packet[20 .. 23]);
      $p5Score = bc_enc::dece_int(@packet[24 .. 27]);
      $p1ID = bc_enc::dece_short(@packet[28 .. 29]);
      $p2ID = bc_enc::dece_short(@packet[30 .. 31]);
      $p3ID = bc_enc::dece_short(@packet[32 .. 33]);
      $p4ID = bc_enc::dece_short(@packet[34 .. 35]);
      $p5ID = bc_enc::dece_short(@packet[36 .. 37]);
    }

    case 0x25	{ # Toggle UFO ship
      my @packet = @_;
      my $available;

      $available = $packet[1];
    }

    case 0x26	{} # Unknown

    case 0x27	{ # Keep alive
    }

    case 0x28	{ # Small position packet
      my @packet = @_;
      my $lastel;
      my ($direction, $timeStamp, $XPixels, $ping, $bounty, $playerID, $toggleables, $YVelocity, $YPixels, $XVelocity, $energy, $lag, $timers, $items);

      $lastel = $#packet;

      $direction = $packet[1];
      $timeStamp = bc_enc::dece_short(@packet[2 .. 3]);
      $XPixels = bc_enc::dece_short(@packet[4 .. 5]);
      $ping = $packet[6];
      $bounty = $packet[7];
      $playerID = $packet[8];
      $toggleables = $packet[9];
      $YVelocity = bc_enc::dece_short(@packet[10 .. 11]);
      $YPixels = bc_enc::dece_short(@packet[12 .. 13]);
      $XVelocity = bc_enc::dece_short(@packet[14 .. 15]);
      if ($lastel >= 16) {
	$energy = bc_enc::dece_short(@packet[16 .. 17]);
	$lag = bc_enc::dece_short(@packet[18 .. 19]);
	$timers = bc_enc::dece_short(@packet[20 .. 21]);
	$items = bc_enc::dece_int(@packet[22 .. 25]);
      }
    }

    case 0x29	{ # Server Map Checksum
      my @packet = @_;
      my ($mapFile, $mapChecksum);

      $mapFile = pack("C*", @packet[1 .. 16]);
      $mapChecksum = bc_enc::dece_int(@packet[17 .. 20]);
    }

    case 0x2A	{} # Server Map File

    case 0x2B	{ # Set personal koth timer
      my @packet = @_;
      my $timer;

      $timer = bc_enc::dece_int(@packet[1 .. 4]);
    }

    case 0x2C	{ # KotH Restarted
      my @packet = @_;
      my ($addingKothTimer, $timerValue, $playerID);

      $addingKothTimer = $packet[1];
      $timerValue = bc_enc::dece_int(@packet[2 .. 5]);
      $playerID = bc_enc::dece_short(@packet[6 .. 7]);
    }

    case 0x2D	{} # Unknown

    case 0x2E	{ # Power-ball position update
      my @packet = @_;
      my ($powerballID, $XPixels, $YPixels, $XVelocity, $YVelocity, $ownerID, $timeStamp);

      $powerballID = $packet[1];
      $XPixels = bc_enc::dece_short(@packet[2 .. 3]);
      $YPixels = bc_enc::dece_short(@packet[4 .. 5]);
      $XVelocity = bc_enc::dece_short(@packet[6 .. 7]);
      $YVelocity = bc_enc::dece_short(@packet[8 .. 9]);
      $ownerID = bc_enc::dece_short(@packet[10 .. 11]);
      $timeStamp = bc_enc::dece_int(@packet[12 .. 15]);
    }

    case 0x2F	{ # Arena directory listing
      my @packet = @_;
      my ($size, $lastel, $termLoc);
      my ($arenaName, $arenaPopulation);

      $lastel = $#packet;
      $arenaName = "";
      $size = 1;
      while ($lastel >= $size) {
	foreach (@packet[$size .. $lastel]) {
	  if ($_ != 0) {
	    $arenaName = $arenaName.pack("C", $_);
	  } else {
	    $termLoc = length($arenaName) + $size;
	    $arenaPopulation = bc_enc::dece_short(@packet[$termLoc+1 .. $termLoc+2]);
	    # $arenaName, $arenaPopulation is here
	    # print "$arenaName - $arenaPopulation\n";

	    last;
	  }
	}
	$arenaName = "";
	$size = $termLoc + 3;
      }
    }

    case 0x30	{} # Got zone banner advertisements

    case 0x31	{ # Got password, go ahead
    }

    # Continuum  
    case 0x32	{ # Change personal ship coordinates
      my @packet = @_;
      my ($XTiles, $YTiles);

      $XTiles = bc_enc::dece_short(@packet[1 .. 2]);
      $YTiles = bc_enc::dece_short(@packet[3 .. 4]);
    }

    case 0x33	{ # Custom login failure message
      my @packet = @_;
      my $message;

      $message = "";
      foreach (@packet[1 .. $#packet]) {
	if ($_ != 0) {
	  $message = $message.pack("C", $_);
	} else {
	  last;
	}
      }
    }

    case 0x34	{ # Continuum version packet
      my @packet = $_;
      my ($version, $checksum);

      $version = bc_enc::dece_short(@packet[1 .. 2]);
      $checksum = bc_enc::dece_int(@packet[3 .. 6]);
    }

    else	{

    }
  }
}

sub command {
  use Switch;
  my ($type, $playerID, $command) = @_;
  my $rank;

  $rank = bc_permission::get_rank($playerID);
  switch($rank) {
    case 0 { # Bot sysops
      sysops($type, $playerID, $command);
      mods($type, $playerID, $command);
      other($type, $playerID, $command);
    }

    case 1 { # Bot mods
      mods($type, $playerID, $command);
      other($type, $playerID, $command);
    }

    case 2 { # Everyone
      other($type, $playerID, $command);
    }

    else {
    }
  }
}

sub sysops {
  my ($type, $playerID, $command) = @_;

}

sub mods {
  my ($type, $playerID, $command) = @_;
  if ($command =~ /^!hi/) {
    bc_packetgen::send_chat_message(5, $playerID, "Hi");
  }
  
}

sub other {
  my ($type, $playerID, $command) = @_;
}

# Signal EOF
1;
